#ifndef __RDMACLIENTREPENDPOINT__
#define __RDMACLIENTREPENDPOINT__

#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <netdb.h>
#include <iostream>
#include <errno.h>
#include <arpa/inet.h>
#include <mutex>
#include <atomic>
#include <shared_mutex>
#include <queue>
#include <rocksdb/db.h>

#include "RdmaRepEndpoint.hpp"
#include "Logger.hpp"
#include "MessageFormats.hpp"

class RdmaClientRepEndpoint : public RdmaRepEndpoint
{
    static int CONN_STATE_INITIALIZED;
    static int CONN_STATE_ADDR_RESOLVED;
    static int CONN_STATE_ROUTE_RESOLVED;
    static int CONN_STATE_RESOURCES_ALLOCATED;
    static int CONN_STATE_CONNECTED;
    static int CONN_STATE_PARTIAL_CLOSED;
    static int CONN_STATE_CLOSED;
    int _state{0};
   
    int _maxInLine{0};
    int _timeoutMs{0};
    int _retryCount{0};
    int _maxRetryCount{2};

    const char *_connData;
    
    void registerMemory();

public:
    
    std::atomic<uint64_t> _requestId{12};

    RdmaClientRepEndpoint(struct rdma_cm_id *id, int sendQueueSize, int recvQueueSize,
                    int sendMsgSize, int recvMsgSize, int maxInLine, int timeout, rocksdb::DB *_db);
    void connect();
    void connect(const char *ip, const char *port, const char *connData);
    bool isConnected();
    void processCmEvent(struct rdma_cm_event *event);
    void createResources(struct ibv_cq *cq);
    void close();
};
#endif