#ifndef __RDMAREPENDPOINT__
#define __RDMAREPENDPOINT__

#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <netdb.h>
#include <iostream>
#include <errno.h>
#include <arpa/inet.h>
#include <mutex>
#include <atomic>
#include <shared_mutex>
#include <queue>
#include <rocksdb/db.h>

#include "Logger.hpp"
#include "MessageFormats.hpp"

class RdmaRepEndpoint
{
    public:

    struct rdma_cm_id *_cm_id{NULL};
    struct ibv_pd *_protectionDomain;

    int _sendQueueSize;
    int _recvQueueSize;
    int _sendMsgSize;
    int _recvMsgSize;
    char *_sendBuff = NULL;
    char *_recvBuff = NULL;
    struct ibv_mr *_sendMr = NULL;
    struct ibv_mr *_recvMr = NULL;
    std::queue<char *> _sendBuffers;
    std::mutex _sendBuffersM;
    rocksdb::DB *_db;

    
    void connect();
    void registerMemory();
    
public:
    
    std::atomic<uint64_t> _requestId{12};

    RdmaRepEndpoint(struct rdma_cm_id *id, int sendQueueSize, int recvQueueSize,
                    int sendMsgSize, int recvMsgSize, rocksdb::DB *_db);
    void processSendCompletion(struct ibv_wc *data);
    void processRecvCompletion(struct ibv_wc *data);
    void processDelete(struct MessageHeader *req);
    void processGet(struct MessageHeader *req);
    void processPut(struct MessageHeader *req);
    int sendMessage(const char *buffer, uint32_t size);
};
#endif