#ifndef __RDMAREPENDPOINTGROUP__
#define __RDMAREPENDPOINTGROUP__

#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <errno.h>
#include <unordered_map>
#include <rocksdb/db.h>

#include "RdmaCqProcessor.hpp"
#include "RdmaRepEndpoint.hpp"
#include "RdmaClientRepEndpoint.hpp"
#include "Executor.hpp"

class RdmaRepEndpointGroup
{
    RdmaCqProcessor *_cqProcessor = NULL;
    // struct rdma_cm_id *_cm_id;

    int _sendQueueSize;
    int _recvQueueSize;
    int _compQueueSize;
    int _sendMsgSize;
    int _recvMsgSize;
    int _maxInLine;
    int _timeoutMs;

    struct rdma_event_channel *_eventChannel;
    std::thread *_cmEventThread;
    bool _stopCMThread;

    Executor *_executor;
     rocksdb::DB *_db;

public:
    std::unordered_map<uint32_t, RdmaRepEndpoint *> *_qpRepEndpointMap;
    RdmaRepEndpointGroup(int sendQueueSize, int recvQueueSize, int compQueueSize, int sendMsgSize,
                         int recvMsgSize, int maxInLine, int timeout, rocksdb::DB *db);
    void processCmEvents();
    void processCmEvent(struct rdma_cm_event *event);
    void startCmProcessor();
    void setExecutor(Executor *ex);
    RdmaClientRepEndpoint *createEndpoint();
    void close();
    struct ibv_cq *createCq(struct rdma_cm_id *id);
};
#endif