#ifndef __RdmaSalEndpoint__
#define __RdmaSalEndpoint__

#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <errno.h>
#include <iostream>
#include <rocksdb/db.h>
#include "RdmaEndpoint.hpp"
#include "MessageFormats.hpp"
#include "Logger.hpp"

class RdmaSalEndpoint 
{
public:
    static int CONN_STATE_INITIALIZED;
    static int CONN_STATE_RESOURCES_ALLOCATED;
    static int CONN_STATE_CONNECTED;
    static int CONN_STATE_CLOSED;
    int _state{0};

    struct rdma_cm_id *_cm_id{NULL};
    struct ibv_cq *_completionQueue{NULL};
    struct ibv_pd *_protectionDomain{NULL};

    int _sendQueueSize{0};
    int _recvQueueSize{0};
    int _sendMsgSize{0};
    int _recvMsgSize{0};
    char *_sendBuff{NULL};
    char *_recvBuff{NULL};
    struct ibv_mr *_sendMr{NULL};
    struct ibv_mr *_recvMr{NULL};
    std::queue<char*> _sendBuffers;
    std::mutex _sendBuffersM;

    rocksdb::DB *_db;

    RdmaSalEndpoint(struct rdma_cm_id *id, struct ibv_cq *completionQueue, int sendQueueSize,
                    int recvQueueSize, int sendMsgSize, int recvMsgSize,rocksdb::DB *_db);
    void createResources();
    void processCmEvent(struct rdma_cm_event *event);
    void processCqEvent(struct ibv_wc wc);
    void processSendCompletion(struct ibv_wc *data);
    void processRecvCompletion(struct ibv_wc *data);
    void processDelete(struct MessageHeader *req);
    void processGet(struct MessageHeader *req);
    void processPut(struct MessageHeader *req);
    int sendMessage(const char *buffer, uint32_t size);
    void close();
};
#endif