#ifndef __RdmaServerEndpointGroup__
#define __RdmaServerEndpointGroup__

#include <rdma/rdma_cma.h>
#include <rdma/rdma_verbs.h>
#include <stdint.h>
#include <thread>
#include <netdb.h>
#include <arpa/inet.h>
#include <map>
#include <vector>
#include <mutex>
#include <shared_mutex>

#include "RdmaSalEndpoint.hpp"
#include "RdmaServerRepEndpoint.hpp"
#include "RdmaEndpointGroup.hpp"
#include "CqEventData.hpp"
#include "Executor.hpp"
#include "RdmaCmProcessor.hpp"
#include "RdmaCqProcessor.hpp"
#include "Logger.hpp"

class RdmaServerEndpointGroup
{
    /*
     * Variables to maintain Group state
     */
    static int CONN_STATE_INITIALIZED;
    static int CONN_STATE_BINDED;
    static int CONN_STATE_CONNECTED;
    static int CONN_STATE_CLOSED;

    RdmaCqProcessor *_cqProcessor{NULL};

    Executor *_executor;

    
    struct rdma_cm_id *_cm_id{NULL};
    struct rdma_event_channel *_eventChannel{NULL};
    std::thread *_cmEventThread{NULL};
    bool _stopCmProcessor{false};

    /*
     * variables to maintain queue state
     */
    int _sendQueueSize{0};
    int _recvQueueSize{0};
    int _compQueueSize{0};
    int _sendMsgSize{0};
    int _recvMsgSize{0};
    rocksdb::DB *_db;

    mutable std::shared_mutex _salMutex;
    mutable std::shared_mutex _repMutex;

    void clientClose();

public:
    std::unordered_map<uint32_t, RdmaSalEndpoint *> *_qpSalEndpointMap;
    std::unordered_map<uint32_t, RdmaRepEndpoint*> *_qpRepEndpointMap;

    RdmaServerEndpointGroup(int sendQueueSize, int recvQueueSize, int compQueueSize,
                            int sendMsgSize, int recvMsgSize, rocksdb::DB *db);
    // void setExecutor(Executor *executor);
    void bind(const char *ip, const char *port, int backlog);

    struct ibv_cq *createCq(struct rdma_cm_id *id);
    void dispatchCqEvents(ibv_wc wc[], int size);
    void setExecutor(Executor *ex);
    void processCmEvents();
    void startCmProcessor(bool newThread);
    void processCmEvent(struct rdma_cm_event *event);
    void createEpCmEvent(struct rdma_cm_event *event);
    void close();
};

#endif