#include "RdmaRepEndpointGroup.hpp"

RdmaRepEndpointGroup::RdmaRepEndpointGroup(int sendQueueSize, int recvQueueSize, int compQueueSize, int sendMsgSize,
                                           int recvMsgSize, int maxInLine, int timeout, rocksdb::DB *db)
    : _sendQueueSize(sendQueueSize), _recvQueueSize(recvQueueSize), _compQueueSize(compQueueSize), _sendMsgSize(sendMsgSize),
      _recvMsgSize(recvMsgSize), _maxInLine(maxInLine), _timeoutMs(timeout), _db(db)
{
    std::cout << "RdmaRepEndpointGroup : Step 1 creating event channel" << std::endl;
    _eventChannel = rdma_create_event_channel();
    _stopCMThread = false;
    if (_eventChannel == NULL)
    {
        std::cout << "RdmaRepEndpointGroup : error creating event channel";
    }
    _qpRepEndpointMap = new std::unordered_map<uint32_t, RdmaRepEndpoint *>();
}

void RdmaRepEndpointGroup::startCmProcessor()
{
    _cmEventThread = new std::thread(&RdmaRepEndpointGroup::processCmEvents, this);
    pthread_setname_np(_cmEventThread->native_handle(), "RepCMProcessor");
}

struct ibv_cq *RdmaRepEndpointGroup::createCq(struct rdma_cm_id *id)
{
    if (_cqProcessor == NULL)
    {
        CPPLog::LOG_ALWAYS("RdmaServerEndpointGroup : step 5 create repcq processor");
        _cqProcessor = new RdmaCqProcessor(_executor, id->verbs, _compQueueSize);
        _cqProcessor->start();
    }
    return _cqProcessor->getCq();
}

void RdmaRepEndpointGroup::processCmEvents()
{
    int ret;
    struct rdma_cm_event *event;
    std::cout << "RdmaRepEndpointGroup : starting cm processing thread" << std::endl;
    while (!_stopCMThread)
    {
        ret = rdma_get_cm_event(_eventChannel, &event);
        if (ret)
        {
            std::cout << "RdmaRepEndpointGroup : rdma_get_cm_event failed" << std::endl;
            continue;
        }
        processCmEvent(event);
        ret = rdma_ack_cm_event(event);
        if (ret)
        {
            std::cout << "RdmaRepEndpointGroup : rdma_ack_cm_event failed";
        }
    }
}

void RdmaRepEndpointGroup::processCmEvent(struct rdma_cm_event *event)
{
    std::cout << "RdmaRepEndpointGroup :  event" << rdma_event_str(event->event) << std::endl;
    if (event->id != NULL && event->id->context != NULL)
    {
        if (event->event == RDMA_CM_EVENT_ADDR_RESOLVED && event->id != NULL)
        {
            ((RdmaClientRepEndpoint *)event->id->context)->createResources(createCq(event->id));
        }
        ((RdmaClientRepEndpoint *)event->id->context)->processCmEvent(event);
        if (event->event == RDMA_CM_EVENT_ADDR_RESOLVED)
        {
            _qpRepEndpointMap->emplace(event->id->qp->qp_num, ((RdmaRepEndpoint *)event->id->context));
        }
        if (event->event == RDMA_CM_EVENT_DISCONNECTED)
        {
            if (_qpRepEndpointMap->find(event->id->qp->qp_num) != _qpRepEndpointMap->end())
                _qpRepEndpointMap->erase(event->id->qp->qp_num);
            delete ((RdmaRepEndpoint *)event->id->context);
        }
    }
    else
    {
        std::cout << "RdmaRepEndpointGroup : Not able to procces CM EVent";
        std::cout << rdma_event_str(event->event) << event->id << " ";
        std::cout << event->listen_id << std::endl;
    }
}

RdmaClientRepEndpoint *RdmaRepEndpointGroup::createEndpoint()
{
    struct rdma_cm_id *id = NULL;
    int ret = rdma_create_id(_eventChannel, &id, NULL, RDMA_PS_TCP);
    if (ret == -1)
        std::cout << "CMProcesor : rdma_create_id failed" << std::endl;
    RdmaClientRepEndpoint *endpoint = new RdmaClientRepEndpoint(id, _sendQueueSize, _recvQueueSize, _sendMsgSize,
                                                                _recvMsgSize, _maxInLine, _timeoutMs, _db);
    id->context = (void *)endpoint;
    return endpoint;
}

void RdmaRepEndpointGroup::setExecutor(Executor *ex)
{
    _executor = ex;
}

void RdmaRepEndpointGroup::close()
{
    _stopCMThread = true;
    _cmEventThread->join();
    _cqProcessor->close();
    delete _cmEventThread;
    delete _cqProcessor;
    delete _qpRepEndpointMap;
    rdma_destroy_event_channel(_eventChannel);
}