#include <iostream>
#include "RdmaServerEndpointGroup.hpp"
#include "RdmaRepEndpointGroup.hpp"
#include "RdmaEndpoint.hpp"
#include "Executor.hpp"
#include "Properties.hpp"
#include "rocksdb/db.h"
#include "rocksdb/slice.h"
#include "rocksdb/options.h"

int connectToRepServer(Properties &prop, RdmaRepEndpointGroup* repGroup)
{
    int followers = stoi(prop.getValue("FOLLOWERS"));
    std::cout<<"followers "<<followers<<"\n";
    RdmaClientRepEndpoint* clientEPs[followers];
    for(int i = 0 ;i< followers ;i++)
    {
        clientEPs[i] = repGroup->createEndpoint();
        std::string ip = prop.getValue("FOLLOWER"+std::to_string(i+1)+"_IP");
        std::string port = prop.getValue("FOLLOWER"+std::to_string(i+1)+"_PORT");
        std::cout<<"Connecting to follower "<<ip<<":"<<port<<"\n";
        clientEPs[i]->connect(ip.c_str(), port.c_str(), "fol");
    }
   return 0;
}
int main()
{
    std::cout << "Starting Server Main Thread\n";
    Properties prop("prop.config");
    int sendQS = stoi(prop.getValue("sendQS"));
    int recvQS = stoi(prop.getValue("recvQS"));
    int compQS = stoi(prop.getValue("compQS"));
    int sendMS = stoi(prop.getValue("sendMS"));
    int recvMS = stoi(prop.getValue("recvMS"));
    std::string dbpath = prop.getValue("DB_PATH");
    std::string serverIP = prop.getValue("SERVER_IP");
    std::string serverPort = prop.getValue("SERVER_PORT");
    int executorPoolSize = stoi(prop.getValue("EXECUTOR_POOL_SIZE"));
    int enableLogging = stoi(prop.getValue("ENABLE_LOGGING"));
    if (enableLogging == 0)
        CPPLog::Logger::getInstance()->updateLogLevel(CPPLog::DISABLE_LOG);

    rocksdb::DB *db;
    rocksdb::Options options;
    options.create_if_missing = true;
    // open a database with a name which corresponds to a file system directory
    rocksdb::Status status = rocksdb::DB::Open(options, dbpath, &db);
    if (!status.ok())
    {
        CPPLog::LOG_ERROR(status.ToString().c_str());
        exit(1);
    }
    CPPLog::LOG_ALWAYS("Rocks started");

    Executor *executor = new Executor(executorPoolSize);

    RdmaRepEndpointGroup *repgroup = new RdmaRepEndpointGroup(sendQS, recvQS, compQS, sendMS, recvMS, 1, 100000, db);
    RdmaServerEndpointGroup *sgroup = new RdmaServerEndpointGroup(sendQS, recvQS, compQS, sendMS, recvMS, db);

    executor->createThreads(repgroup->_qpRepEndpointMap, sgroup->_qpRepEndpointMap, sgroup->_qpSalEndpointMap);
    repgroup->setExecutor(executor);
    sgroup->setExecutor(executor);
    repgroup->startCmProcessor();
    sgroup->bind(serverIP.c_str(), serverPort.c_str(), 2);
    std::thread t(connectToRepServer, std::ref(prop), (repgroup));

    sgroup->startCmProcessor(false);

    std::cout << "rhdhj" << std::endl;
    while (1)
        ;
}
